from distutils.core import setup
setup(
    name = "djangogcal",
    packages = ["djangogcal", ],
    version = "0.2.1",
    description = "A Django application allowing developers to synchronise instances of their models with Google Calendar.",
    author = "Incuna Ltd",
    author_email = "admin@incuna.com",
    url = "http://incuna.com/",
)
